@@ -5,9 +5,10 @@
 # Copyright (C) 2002       Masato Taruishi <taru@debian.org>
 # Copyright (C) 2006-2008  Junichi Uekawa <dancer@debian.org>
 # Copyright (C) 2007       Famelis George <famelis@otenet.gr>
-# Copyright (C) 2008-2022  Francesco Poli <invernomuto@paranoici.org>
+# Copyright (C) 2008-2023  Francesco Poli <invernomuto@paranoici.org>
 # Copyright (C) 2009-2010  Ryan Niebur <ryan@debian.org>
 # Copyright (C) 2013       Google Inc
+# Copyright (C) 2015       Michael Gold <michael@bitplane.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -57,8 +58,9 @@ class AppConfig
       sprintf(_(" -S <states>      : Filter bugs by pending-state categories you want to see\n                    [%s].\n"), @stats.join(',')),
       _(" -B <bug#>        : Filter bugs by number, showing only the specified bugs.\n"),
       _(" -D               : Show downgraded packages, too.\n"),
-      sprintf(_(" -H <hostname>    : Hostname of Debian Bug Tracking System [%s].\n"), @hostname),
-      sprintf(_(" -p <port>        : Port number of the server [%s].\n"), @port),
+      sprintf(_(" -u <url>         : SOAP URL for Debian Bug Tracking System\n                    [%s].\n"), @soapurl),
+      _(" -H <hostname>    : Hostname of Debian Bug Tracking System\n                    (for http, deprecated).\n"),
+      _(" -p <port>        : Port number of the server\n                    (for http, deprecated).\n"),
       sprintf(_(" -P <priority>    : Pin-Priority value [%s].\n"), @pin_priority),
       _(" -E <title>       : Title of RSS output.\n"),
       _(" -q               : Don't display progress bar.\n"),
@@ -92,6 +94,7 @@ class AppConfig
                 ["done", _("Resolved in some Version")]]
     @fbugs = nil
     @show_downgrade = false
+    @soapurl = "https://bugs.debian.org:443/cgi-bin/soap.cgi"
     @hostname = "bugs.debian.org"
     @port = 80
     @querystep = 200
@@ -130,14 +133,16 @@ class AppConfig
     if Locale.charset == "UTF-8"
       @arrow = "→"
     end
+
+    @debugfile = $stderr
   end
 
   attr_accessor :severity, :stats, :quiet, :title
-  attr_accessor :show_downgrade, :hostname, :tag, :fbugs, :querystep, :parsestep
+  attr_accessor :show_downgrade, :tag, :fbugs, :querystep, :parsestep
   attr_accessor :frontend, :pin_priority, :yes, :ignore_regexp, :force_pin
   attr_accessor :default_reply
   attr_reader :command, :parser, :querybts, :ignore_bugs, :system_ignore_bugs
-  attr_reader :browser, :arrow, :cmd_prefix, :ext_user
+  attr_reader :browser, :arrow, :cmd_prefix, :ext_user, :debugfile
 
   def parse_options
     all_severities = ["critical","grave","serious","important","normal","minor","wishlist"]
@@ -149,6 +154,10 @@ class AppConfig
       end
     end
 
+    if /soap_url='(.*)'/ =~ `apt-config #{@apt_conf} shell soap_url AptListbugs::URL`
+      @soapurl = $1
+    end
+
     if /qb='(.*)'/ =~ `apt-config #{@apt_conf} shell qb AptListbugs::QueryStep`
       @querystep = $1.to_i if $1.to_i > 0
     end
@@ -165,6 +174,7 @@ class AppConfig
                            ['--stats', '-S', GetoptLong::REQUIRED_ARGUMENT],
                            ['--bugs', '-B', GetoptLong::REQUIRED_ARGUMENT],
                            ['--show-downgrade', '-D', GetoptLong::NO_ARGUMENT],
+                           ['--url', '-u', GetoptLong::REQUIRED_ARGUMENT],
                            ['--hostname', '-H', GetoptLong::REQUIRED_ARGUMENT],
                            ['--port', '-p', GetoptLong::REQUIRED_ARGUMENT],
                            ['--pin-priority', '-P', GetoptLong::REQUIRED_ARGUMENT],
@@ -203,10 +213,17 @@ class AppConfig
           @fbugs = optargs.split(',')
         when '--show-downgrade'
           @show_downgrade = true
+        when '--url'
+          @soapurl = optargs
         when '--hostname'
           @hostname = optargs
+          @soapurl = "http://#{@hostname}:#{@port}/cgi-bin/soap.cgi"
+          # TRANSLATORS: "W: " is a label for warnings; you may translate it with a suitable abbreviation of the word "warning"
+          $stderr.puts _("W: ") + sprintf(_("%s IS DEPRECATED. USE --url INSTEAD"), "--hostname")
         when '--port'
           @port = optargs.to_i
+          @soapurl = "http://#{@hostname}:#{@port}/cgi-bin/soap.cgi"
+          $stderr.puts _("W: ") + sprintf(_("%s IS DEPRECATED. USE --url INSTEAD"), "--port")
         when '--pin-priority'
           @pin_priority = optargs
         when '--title'
@@ -238,6 +255,18 @@ class AppConfig
       exit 1
     end
 
+    if $DEBUG
+      if /df='(.*)'/ =~ `apt-config #{@apt_conf} shell df AptListbugs::DebugFile`
+        df = $1
+        begin
+          @debugfile = File.open(df, "w")
+        rescue Errno::EACCES
+          $stderr.puts _("W: ") + sprintf(_("Cannot write to %s"), df)
+          @debugfile = $stderr
+        end
+      end
+    end
+
     if ! $stdout.isatty
       @quiet = true
       @yes = false if @yes.nil?
@@ -248,7 +277,6 @@ class AppConfig
 
     # warn the user about unknown (possibly misspelled) severities
     (@severity - all_severities).each { |unrec|
-      # TRANSLATORS: "W: " is a label for warnings; you may translate it with a suitable abbreviation of the word "warning"
       $stderr.puts _("W: ") + sprintf(_("Unrecognized severity '%s' will be ignored by the Debian BTS."), unrec)
     }
 
@@ -266,9 +294,9 @@ class AppConfig
 
     # proxy settings in apt.conf
     if /http_proxy='(.*)'/ =~ `apt-config #{@apt_conf} shell http_proxy acquire::http::proxy`
-      $stderr.puts "proxy configuration from apt.conf: #{$1}" if $DEBUG
+      @debugfile.puts "proxy configuration from apt.conf: #{$1}" if $DEBUG
       if $1 == 'DIRECT' || $1 == ''
-        $stderr.puts "Disabling proxy due to DIRECT, or empty string" if $DEBUG
+        @debugfile.puts "Disabling proxy due to DIRECT, or empty string" if $DEBUG
         ENV.delete("http_proxy")
         ENV.delete("soap_use_proxy")
       else
@@ -277,24 +305,24 @@ class AppConfig
       end
     end
     if /proxy_detect='(.*)'/ =~ `apt-config #{@apt_conf} shell proxy_detect acquire::http::proxy-auto-detect`
-      $stderr.puts "auto proxy detect command from apt.conf: #{$1}" if $DEBUG
+      @debugfile.puts "auto proxy detect command from apt.conf: #{$1}" if $DEBUG
       if FileTest.executable?("#{$1}")
         detected_proxy = `#{$1}`.chomp
-        $stderr.puts "auto detected proxy: #{detected_proxy}" if $DEBUG
+        @debugfile.puts "auto detected proxy: #{detected_proxy}" if $DEBUG
         if /^http/ =~ detected_proxy
           ENV["http_proxy"] = detected_proxy
           ENV["soap_use_proxy"] = "on"
         else
-          $stderr.puts "Ignoring auto detected proxy not beginning with 'http'" if $DEBUG
+          @debugfile.puts "Ignoring auto detected proxy not beginning with 'http'" if $DEBUG
         end
       else
         $stderr.puts _("W: ") + sprintf(_("Cannot execute auto proxy detect command %s"), $1)
       end
     end
     if /http_proxy='(.*)'/ =~ `apt-config #{@apt_conf} shell http_proxy acquire::http::proxy::bugs.debian.org`
-      $stderr.puts "proxy configuration from apt.conf, specific for bugs.debian.org: #{$1}" if $DEBUG
+      @debugfile.puts "proxy configuration from apt.conf, specific for bugs.debian.org: #{$1}" if $DEBUG
       if $1 == 'DIRECT'
-        $stderr.puts "Disabling proxy due to DIRECT" if $DEBUG
+        @debugfile.puts "Disabling proxy due to DIRECT" if $DEBUG
         ENV.delete("http_proxy")
         ENV.delete("soap_use_proxy")
       else
@@ -323,7 +351,7 @@ class AppConfig
     end
 
     @parser =
-      Debian::BTS::Parser::SoapIndex.new(@hostname, @port)
+      Debian::BTS::Parser::SoapIndex.new(@soapurl, @debugfile)
 
     if FileTest.executable?("#{QUERYBTS}")
       @querybts = QUERYBTS
@@ -489,7 +517,7 @@ class Viewer
             answer = "n"
           end
         else
-          answer = a.downcase
+          answer = a.downcase.strip
         end
         case answer
 
@@ -516,10 +544,10 @@ class Viewer
               bugnum = $2
             end
             if ! bugnum.nil?
-              $stderr.puts "Invoking querybts for ##{bugnum}" if $DEBUG
+              @config.debugfile.puts "Invoking querybts for ##{bugnum}" if $DEBUG
               querybtscommandline = "#{@config.cmd_prefix}#{@config.querybts} -u text #{bugnum}"
               if system(querybtscommandline)
-                $stderr.puts "successfully invoked querybts" if $DEBUG
+                @config.debugfile.puts "successfully invoked querybts" if $DEBUG
               else
                 $stderr.puts _("W: ") + _("Failed to invoke querybts.")
                 $stderr.puts " #{querybtscommandline}"
@@ -622,16 +650,16 @@ class Viewer
              bugnames.size) % {:nbugs => bugnames.size,
                                :blist => bugnames.join(', ')}
               h, e = pinned(pkgs.keys, cur_pkgs, feared_bugs)
+              @dodged_bugnums.concat(e) if e != nil
             end
           end
-          @dodged_bugnums.concat(e) if e != nil
 
         when "c"
-          $stderr.puts bugs if $DEBUG
+          @config.debugfile.puts bugs if $DEBUG
           display_bugs_as_html(bugs, cur_pkgs, new_pkgs, "c")
 
         when "w"
-          $stderr.puts bugs if $DEBUG
+          @config.debugfile.puts bugs if $DEBUG
           if @config.browser != nil
             display_bugs_as_html(bugs, cur_pkgs, new_pkgs)
           else
@@ -663,8 +691,8 @@ class Viewer
              pkgs.size) % {:npkgs => pkgs.size,
                            :plist => pkgs.keys.join(', ')}
               h, e = pinned(pkgs.keys, cur_pkgs, bugs)
+              @hold_pkg_keys.concat(h) if h != nil
             end
-            @hold_pkg_keys.concat(h) if h != nil
             save(bugs) if @config.force_pin
           else
             @config.frontend.puts sprintf(_("All selected packages are already pinned. Ignoring %s command."), answer)
@@ -781,7 +809,7 @@ class Viewer
           holdstr << "\nPin-Priority: #{pin_pri}\n"
         end
       }
-      $stderr.puts holdstr if $DEBUG
+      @config.debugfile.puts holdstr if $DEBUG
       if holdstr != ""
         @pref_str << holdstr
         # TRANSLATORS: %{packgl} is a list of packages.
@@ -928,10 +956,10 @@ class Viewer
           _("You can view the bug lists in HTML at the following URI:\n") +
           "file://" + tmp.path + "\n"
       when "w"
-        $stderr.puts "Invoking browser for #{tmp.path}" if $DEBUG
+        @config.debugfile.puts "Invoking browser for #{tmp.path}" if $DEBUG
         browsercommandline = "#{@config.cmd_prefix}#{@config.browser} #{tmp.path}"
         if system(browsercommandline)
-          $stderr.puts "successfully invoked browser" if $DEBUG
+          @config.debugfile.puts "successfully invoked browser" if $DEBUG
         else
           $stderr.puts _("W: ") + _("Failed to invoke browser.")
           $stderr.puts " #{browsercommandline}"
@@ -1060,15 +1088,15 @@ module Factory
       # TRANSLATORS: this sentence, followed by the translation of "Done" (see above) should fit in less than 79 columns to work well with default width terminals
       yield _("Retrieving bug reports..."), "0%"
       begin
-        $stderr.puts " Using QueryStep = #{config.querystep}" if $DEBUG
-        $stderr.puts " Using ParseStep = #{config.parsestep}" if $DEBUG
+        config.debugfile.puts " Using QueryStep = #{config.querystep}" if $DEBUG
+        config.debugfile.puts " Using ParseStep = #{config.parsestep}" if $DEBUG
         # obtain the array of bugs
         bugs = config.parser.parse(ma_copies, config.querystep, config.parsestep, config.severity) { |pct|
           yield _("Retrieving bug reports..."), pct
         }
       rescue SOAP::HTTPStreamError => exception
         config.frontend.puts _(" Fail")
-        $stderr.puts " Exception: " + exception.class.to_s if $DEBUG
+        config.debugfile.puts " Exception: " + exception.class.to_s if $DEBUG
         $stderr.puts _("E: ") + _("HTTP GET failed")
         retrycount -= 1
         if config.frontend.yes_or_no?(_("Retry downloading bug information?")) && retrycount > 0
@@ -1080,7 +1108,7 @@ module Factory
         bugs = []
       rescue SOAP::EmptyResponseError => exception
         config.frontend.puts _(" Fail")
-        $stderr.puts " Exception: " + exception.class.to_s if $DEBUG
+        config.debugfile.puts " Exception: " + exception.class.to_s if $DEBUG
         $stderr.puts _("E: ") + _("Empty stream from SOAP")
         retrycount -= 1
         if config.frontend.yes_or_no?(_("Retry downloading bug information?")) && retrycount > 0
@@ -1092,7 +1120,7 @@ module Factory
         bugs = []
       rescue Exception => exception
         config.frontend.puts _(" Fail")
-        $stderr.puts " Exception: " + exception.class.to_s if $DEBUG
+        config.debugfile.puts " Exception: " + exception.class.to_s if $DEBUG
         config.frontend.puts _("Error retrieving bug reports from the server with the following error message:")
         $stderr.puts _("E: ") + "#{$!}"
         if exception.kind_of? SocketError
@@ -1123,7 +1151,7 @@ module Factory
     end
 
     def delete_regexp_bugs(bugs, regexp)
-      $stderr.puts "Ignoring regexp: #{regexp}" if $DEBUG
+      config.debugfile.puts "Ignoring regexp: #{regexp}" if $DEBUG
       bugs.delete_if { |bug| bug.desc =~ /#{config.ignore_regexp}/ }
     end
 
@@ -1190,7 +1218,7 @@ module Factory
       # find out if this version is buggy or not depending on the
       # fixed / found arrays.
 
-      $stderr.puts " .. checking ver #{ver} against fixed:#{fixed} found:#{found}" if $DEBUG
+      config.debugfile.puts " .. checking ver #{ver} against fixed:#{fixed} found:#{found}" if $DEBUG
 
       fixed_max_below = nil
       found_max_below = nil
@@ -1214,7 +1242,7 @@ module Factory
         if found_min_above == nil
           # no new version found;
           # which I guess means that the BTS is missing version info
-          $stderr.puts " ... no found info: I guess I am buggy" if $DEBUG
+          config.debugfile.puts " ... no found info: I guess I am buggy" if $DEBUG
           val=true
         else
           # found_min_above is not nil, which means I may not be buggy;
@@ -1222,27 +1250,27 @@ module Factory
           # which probably means 'found' info is missing from BTS
           if fixed_min_above == nil ||
               Debian::Dpkg.compare_versions(fixed_min_above, "gt", found_min_above)
-            $stderr.puts " ... bug found in a later version: I guess I am not buggy" if $DEBUG
+            config.debugfile.puts " ... bug found in a later version: I guess I am not buggy" if $DEBUG
             val=false
           else
-            $stderr.puts " ... bug fixed in a later version before it's found again: I guess I am buggy" if $DEBUG
+            config.debugfile.puts " ... bug fixed in a later version before it's found again: I guess I am buggy" if $DEBUG
             val=true
           end
         end
       else if fixed_max_below == nil
              # the bug was found before (or in) this version, but fixed
              # later (or never): it means I am buggy
-             $stderr.puts " ... bug found in a prior version, but not yet fixed: I am buggy" if $DEBUG
+             config.debugfile.puts " ... bug found in a prior version, but not yet fixed: I am buggy" if $DEBUG
              val=true
            else if found_max_below == nil || Debian::Dpkg.compare_versions(fixed_max_below, "gt", found_max_below)
                   # the bug was not found between the latest fixed version
                   # and this version: it means I am not buggy
-                  $stderr.puts " ... bug not found between the latest fixed version and this version: I am not buggy" if $DEBUG
+                  config.debugfile.puts " ... bug not found between the latest fixed version and this version: I am not buggy" if $DEBUG
                   val=false
                 else
                   # the bug was indeed found between the latest fixed version
                   # and this version: it means I am buggy
-                  $stderr.puts " ... bug found between the latest fixed version and this version: I am buggy" if $DEBUG
+                  config.debugfile.puts " ... bug found between the latest fixed version and this version: I am buggy" if $DEBUG
                   val=true
                 end
            end
@@ -1256,15 +1284,15 @@ module Factory
       # @return false: bug is relevant, true: bug is irrelevant, should be removed.
       val = false
 
-      $stderr.puts "Start checking: #{bug_number}" if $DEBUG
+      config.debugfile.puts "Start checking: #{bug_number}" if $DEBUG
 
       # ignore bugs that have no fixed version info, and are closed with a XXXX-done
       if fixed.nil? && bug_stat == "done"
-        $stderr.puts "bug apparently closed with XXXX-done without version info" if $DEBUG
+        config.debugfile.puts "bug apparently closed with XXXX-done without version info" if $DEBUG
         val = true
       else if new_ver.nil?
              # no specific version was given for the package, which means that we are interested in all (non archived) bugs
-             $stderr.puts "no package version given" if $DEBUG
+             config.debugfile.puts "no package version given" if $DEBUG
              val = false
            else if cur_ver.nil?
                   # no known installed version, which means that we want to check the new version
@@ -1276,7 +1304,7 @@ module Factory
            end
       end
 
-      $stderr.puts "in conclusion, for bug #{bug_number} comparing fixed:[#{fixed}], found:[#{found}], cur_ver:#{cur_ver} and new_ver:#{new_ver} results in removal:#{val}" if $DEBUG
+      config.debugfile.puts "in conclusion, for bug #{bug_number} comparing fixed:[#{fixed}], found:[#{found}], cur_ver:#{cur_ver} and new_ver:#{new_ver} results in removal:#{val}" if $DEBUG
       val
     end
 
@@ -1326,13 +1354,13 @@ module Factory
     end
 
     def delete_unwanted_tag_bugs( bugs )
-      $stderr.puts "checking unwanted bugs: #{bugs}" if $DEBUG
+      config.debugfile.puts "checking unwanted bugs: #{bugs}" if $DEBUG
       bugs.delete_if { |bug|
         val = false
-        $stderr.puts "#{bug}" if $DEBUG
+        config.debugfile.puts "#{bug}" if $DEBUG
         config.tag.each { |tag|
           if bug.tags && bug.tags.include?( tag )
-            $stderr.puts "#{bug} has {tag}" if $DEBUG
+            config.debugfile.puts "#{bug} has {tag}" if $DEBUG
           else
             val = true
           end
@@ -1398,14 +1426,18 @@ class ConsoleFrontend
         msgyn << " [y/N]"
       end
       a = ask msgyn
-      if a == ""
-        return default
-      elsif a == "Y" || a == "y"
-        return true
-      elsif a == "N" || a == "n"
-        return false
-      elsif a.nil?
+      if a.nil?
         $stdout.print "\n"
+      else
+        answer = a.downcase.strip
+        case answer
+        when ""
+          return default
+        when "y"
+          return true
+        when "n"
+          return false
+        end
       end
     end
   end
